==> cascading-cassandra-0.0.5.jar: test/com/ifesdjeen/cascading/cassandra/core_test.clj <==
(ns com.ifesdjeen.cascading.cassandra.core-test
  (:require [clojurewerkz.cassaforte.client :as cc]
            [clojurewerkz.cassaforte.schema :as sch]
            [clojurewerkz.cassaforte.conversion :as cconv]
            [clojurewerkz.cassaforte.cql    :as cql]
            [clojurewerkz.cassaforte.bytes  :as bytes]

            [clojurewerkz.cassaforte.thrift.core :as thrift]
            [clojurewerkz.cassaforte.thrift.column-definition :as cd]
            [clojurewerkz.cassaforte.thrift.column-family-definition :as cfd]
            )
  (:use cascalog.api
        clojure.test
        [midje sweet cascalog])
  (:require [cascalog.io :as io]
            [cascalog.ops :as c])
  (:import [cascading.tuple Fields]
           [cascading.scheme Scheme]
           [com.ifesdjeen.cascading.cassandra CassandraTap CassandraScheme]
           [org.apache.cassandra.utils ByteBufferUtil]
           [org.apache.cassandra.thrift Column]))

(defmacro with-thrift-exception-handling
[& forms]
`(try
   (do ~@forms)
   (catch org.apache.cassandra.thrift.InvalidRequestException ire#
     (println (.getWhy ire#)))))

(cc/connect! "127.0.0.1")
(sch/set-keyspace "CascadingCassandra")

(defn create-test-column-family
  []
  (with-thrift-exception-handling
    (cql/drop-column-family "libraries"))
  (cql/create-column-family "libraries"
                            {:name      "varchar"
                             :language  "varchar"
                             :votes     "int"}
                            :primary-key :name))

(defn create-tap
  ([]
     (create-tap ["name" "language" "votes"] {"name"     "?value1"
                                              "language" "?value2"
                                              "votes"    "?value3"}))
  ([columns mappings]
      (let [keyspace      "CascadingCassandra"
            column-family "libraries"
            scheme        (CassandraScheme. "127.0.0.1"
                                            "9160"
                                            keyspace
                                            column-family
                                            "name"
                                            columns
                                            mappings
                                            {"cassandra.inputPartitioner" "org.apache.cassandra.dht.RandomPartitioner"
                                             "cassandra.outputPartitioner" "org.apache.cassandra.dht.RandomPartitioner"})
            tap           (CassandraTap. scheme)]
        tap)))

(deftest t-cassandra-tap-as-source
  (create-test-column-family)
  (dotimes [counter 100]
    (cql/insert "libraries" {:name (str "Cassaforte" counter) :language (str "Clojure" counter) :votes (int counter)}))

  (fact "Handles simple calculations"
        (<-
         [?count ?sum]
         ((create-tap) ?value1 ?value2 ?value3)
         (c/count ?count)
         (c/sum ?value3 :> ?sum))
        => (produces [[100 4950]])))

(deftest t-cassandra-tap-as-source-2
  (create-test-column-family)

  (cql/insert "libraries" {:name "Riak" :language "Erlang" :votes (int 5)})
  (cql/insert "libraries" {:name "Cassaforte" :language "Clojure" :votes (int 3)})

  (fact "Retrieves data"
        (<-
         [?value1 ?value3]
         ((create-tap) ?value1 ?value2 ?value3))
        => (produces [["Cassaforte" 3] ["Riak" 5]])))


(deftest t-cassandra-tap-as-sink
  (create-test-column-family)
  (let [test-data [["Riak" "Erlang"]
                   ["Cassaforte" "Clojure"]]]

    (?<- (create-tap ["name" "language"] {"name"     "?value1"
                                          "language" "?value2"})
         [?value1 ?value2]
         (test-data ?value1 ?value2))

    (let [res (cconv/to-plain-hash (:rows (cql/execute "SELECT * FROM libraries")))]
      (is (= {:name "Cassaforte" :language "Clojure" :votes nil}
             (get res "Cassaforte")))
      (is (= {:name "Riak" :language "Erlang" :votes nil}
             (get res "Riak"))))))

;;
;;  To whom it may concern, same serialization code, but in Java:
;;
;;  for (IColumn column : columns.values()) {
;;    String name  = ByteBufferUtil.string(column.name());
;;    System.out.print("Name:");
;;    System.out.println(name);
;;
;;    String v = null;
;;
;;    if (name.contains("votes"))
;;      v = String.valueOf(ByteBufferUtil.toInt(column.value()));
;;    else
;;      v = ByteBufferUtil.string(column.value());
;;
;;    System.out.print("Value:");
;;    System.out.println(v);
;;  }

(defmapop deserialize-values
  [columns]
  (into []
        (for [column (.values columns)]
          (let [name (ByteBufferUtil/string (.name column))
                raw-value (.value column)]
            (cond
              (= "votes" name) (ByteBufferUtil/toInt raw-value)
              :else (ByteBufferUtil/string raw-value))))))

(deftest t-cassandra-tap-as-source-3
  (create-test-column-family)

  (cql/insert "libraries" {:name "Riak" :language "Erlang" :votes (int 5)})
  (cql/insert "libraries" {:name "Cassaforte" :language "Clojure" :votes (int 3)})

  (fact "Retrieves data"
        (<-
         [?name ?language ?votes]
         (deserialize-values ?columns :> ?language ?votes)
         ((create-tap [] {}) ?name ?columns))
        => (produces [["Cassaforte" "Clojure" 3] ["Riak" "Erlang" 5]])))



(defmapop deserialize-wide-rows
  [super-columns]
  (into []
        (for [super-column (.values super-columns)]
          (into []
                (for [column (.getSubColumns super-column)]
                  (let [name (ByteBufferUtil/string (.name column))
                        raw-value (.value column)]
                    (ByteBufferUtil/string raw-value)))))))

(deftest t-cassandra-tap-as-source-4-wide-rows
  (with-thrift-exception-handling
    (sch/drop-keyspace "CascadingCassandra"))
  (sch/add-keyspace "CascadingCassandra"
                    "org.apache.cassandra.locator.SimpleStrategy"
                    [(cfd/build-cfd "CascadingCassandra" "libraries" [] :column-type "Super")]
                    :strategy-opts {"replication_factor" "1"})

  (thrift/batch-mutate
   {"key1" {"libraries" {:name1 {:first "a" :second "b"} :name2 {:first "c" :second "d"} :name3 {:first "e" :second "f"}}}
    "key2" {"libraries" {:name1 {:first "g" :second "h"} :name2 {:first "i" :second "j"} :name3 {:first "k" :second "l"}} }}
   (cconv/to-consistency-level :one)
   :type :super)

  (fact "Retrieves data"
        (<-
         [?name1 ?name2 ?name3]
         (deserialize-wide-rows ?super-columns :> ?name1 ?name2 ?name3)
         ((create-tap [] {}) ?name ?super-columns))
        => (produces [[["a" "b"] ["c" "d"] ["e" "f"]]
                      [["g" "h"] ["i" "j"]  ["k" "l"]] ])))

==> cascading-cassandra-0.0.6-20121130.075812-2.jar: test/com/ifesdjeen/cascading/cassandra/core_test.clj <==
(ns com.ifesdjeen.cascading.cassandra.core-test
  (:require [clojurewerkz.cassaforte.client :as cc]
            [clojurewerkz.cassaforte.schema :as sch]
            [clojurewerkz.cassaforte.conversion :as cconv]
            [clojurewerkz.cassaforte.cql    :as cql]
            [clojurewerkz.cassaforte.bytes  :as bytes])
  (:use cascalog.api
        clojure.test
        [midje sweet cascalog])
  (:require [cascalog.io :as io]
            [cascalog.ops :as c])
  (:import [cascading.tuple Fields]
           [cascading.scheme Scheme]
           [com.ifesdjeen.cascading.cassandra CassandraTap CassandraScheme]
           [org.apache.cassandra.utils ByteBufferUtil]
           [org.apache.cassandra.thrift Column]))

(defmacro with-thrift-exception-handling
[& forms]
`(try
   (do ~@forms)
   (catch org.apache.cassandra.thrift.InvalidRequestException ire#
     (println (.getWhy ire#)))))

(cc/connect! "127.0.0.1")
(sch/set-keyspace "CascadingCassandra")

(defn create-test-column-family
  []
  (with-thrift-exception-handling
    (cql/drop-column-family "libraries"))
  (cql/create-column-family "libraries"
                            {:name      "varchar"
                             :language  "varchar"
                             :votes     "int"}
                            :primary-key :name))

(defn create-tap
  ([]
     (create-tap ["name" "language" "votes"] {"name"     "?value1"
                                              "language" "?value2"
                                              "votes"    "?value3"}))
  ([columns mappings]
      (let [keyspace      "CascadingCassandra"
            column-family "libraries"
            scheme        (CassandraScheme. "127.0.0.1"
                                            "9160"
                                            keyspace
                                            column-family
                                            "name"
                                            columns
                                            mappings
                                            {"cassandra.inputPartitioner" "org.apache.cassandra.dht.RandomPartitioner"
                                             "cassandra.outputPartitioner" "org.apache.cassandra.dht.RandomPartitioner"})
            tap           (CassandraTap. scheme)]
        tap)))

(deftest t-cassandra-tap-as-source
  (create-test-column-family)
  (dotimes [counter 100]
    (cql/insert "libraries" {:name (str "Cassaforte" counter) :language (str "Clojure" counter) :votes (int counter)}))

  (fact "Handles simple calculations"
        (<-
         [?count ?sum]
         ((create-tap) ?value1 ?value2 ?value3)
         (c/count ?count)
         (c/sum ?value3 :> ?sum))
        => (produces [[100 4950]])))

(deftest t-cassandra-tap-as-source-2
  (create-test-column-family)

  (cql/insert "libraries" {:name "Riak" :language "Erlang" :votes (int 5)})
  (cql/insert "libraries" {:name "Cassaforte" :language "Clojure" :votes (int 3)})

  (fact "Retrieves data"
        (<-
         [?value1 ?value3]
         ((create-tap) ?value1 ?value2 ?value3))
        => (produces [["Cassaforte" 3] ["Riak" 5]])))


(deftest t-cassandra-tap-as-sink
  (create-test-column-family)
  (let [test-data [["Riak" "Erlang"]
                   ["Cassaforte" "Clojure"]]]

    (?<- (create-tap ["name" "language"] {"name"     "?value1"
                                          "language" "?value2"})
         [?value1 ?value2]
         (test-data ?value1 ?value2))

    (let [res (cconv/to-plain-hash (:rows (cql/execute "SELECT * FROM libraries")))]
      (is (= {:name "Cassaforte" :language "Clojure" :votes nil}
             (get res "Cassaforte")))
      (is (= {:name "Riak" :language "Erlang" :votes nil}
             (get res "Riak"))))))

;;
;;  To whom it may concern, same serialization code, but in Java:
;;
;;  for (IColumn column : columns.values()) {
;;    String name  = ByteBufferUtil.string(column.name());
;;    System.out.print("Name:");
;;    System.out.println(name);
;;
;;    String v = null;
;;
;;    if (name.contains("votes"))
;;      v = String.valueOf(ByteBufferUtil.toInt(column.value()));
;;    else
;;      v = ByteBufferUtil.string(column.value());
;;
;;    System.out.print("Value:");
;;    System.out.println(v);
;;  }

(defmapop deserialize-values
  [columns]
  (into []
        (for [column (.values columns)]
          (let [name (ByteBufferUtil/string (.name column))
                raw-value (.value column)]
            (cond
              (= "votes" name) (ByteBufferUtil/toInt raw-value)
              :else (ByteBufferUtil/string raw-value))))))

(deftest t-cassandra-tap-as-source-3
  (create-test-column-family)

  (cql/insert "libraries" {:name "Riak" :language "Erlang" :votes (int 5)})
  (cql/insert "libraries" {:name "Cassaforte" :language "Clojure" :votes (int 3)})

  (fact "Retrieves data"
        (<-
         [?name ?language ?votes]
         (deserialize-values ?columns :> ?language ?votes)
         ((create-tap [] {}) ?name ?columns))
        => (produces [["Cassaforte" "Clojure" 3] ["Riak" "Erlang" 5]])))
